@@ -16,17 +16,23 @@ from pandas_datareader.bankofcanada import BankOfCanadaReader
 from pandas_datareader.econdb import EcondbReader
 from pandas_datareader.enigma import EnigmaReader
 from pandas_datareader.eurostat import EurostatReader
-from pandas_datareader.exceptions import DEP_ERROR_MSG, ImmediateDeprecationError
+from pandas_datareader.exceptions import (
+    DEP_ERROR_MSG,
+    ImmediateDeprecationError,
+)
 from pandas_datareader.famafrench import FamaFrenchReader
 from pandas_datareader.fred import FredReader
 from pandas_datareader.iex.daily import IEXDailyReader
 from pandas_datareader.iex.deep import Deep as IEXDeep
-from pandas_datareader.iex.tops import LastReader as IEXLasts, TopsReader as IEXTops
+from pandas_datareader.iex.tops import (
+    LastReader as IEXLasts,
+    TopsReader as IEXTops,
+)
 from pandas_datareader.moex import MoexReader
 from pandas_datareader.nasdaq_trader import get_nasdaq_symbols
+from pandas_datareader.naver import NaverDailyReader
 from pandas_datareader.oecd import OECDReader
 from pandas_datareader.quandl import QuandlReader
-from pandas_datareader.robinhood import RobinhoodHistoricalReader, RobinhoodQuoteReader
 from pandas_datareader.stooq import StooqDailyReader
 from pandas_datareader.tiingo import (
     TiingoDailyReader,
@@ -60,8 +66,6 @@ __all__ = [
     "get_iex_book",
     "get_dailysummary_iex",
     "get_data_stooq",
-    "get_data_robinhood",
-    "get_quotes_robinhood",
     "DataReader",
 ]
 
@@ -118,14 +122,6 @@ def get_last_iex(*args, **kwargs):
     return IEXLasts(*args, **kwargs).read()
 
 
-def get_data_robinhood(*args, **kwargs):
-    return RobinhoodHistoricalReader(*args, **kwargs).read()
-
-
-def get_quotes_robinhood(*args, **kwargs):
-    return RobinhoodQuoteReader(*args, **kwargs).read()
-
-
 def get_data_tiingo(*args, **kwargs):
     return TiingoDailyReader(*args, **kwargs).read()
 
@@ -356,11 +352,11 @@ def DataReader(
         "nasdaq",
         "quandl",
         "moex",
-        "robinhood",
         "tiingo",
         "yahoo-actions",
         "yahoo-dividends",
         "av-forex",
+        "av-forex-daily",
         "av-daily",
         "av-daily-adjusted",
         "av-weekly",
@@ -369,6 +365,7 @@ def DataReader(
         "av-monthly-adjusted",
         "av-intraday",
         "econdb",
+        "naver",
     ]
 
     if data_source not in expected_source:
@@ -428,9 +425,12 @@ def DataReader(
             pause=pause,
             session=session,
         ).read()
+
     elif data_source == "stooq":
         return StooqDailyReader(
             symbols=name,
+            start=start,
+            end=end,
             chunksize=25,
             retry_count=retry_count,
             pause=pause,
@@ -515,15 +515,6 @@ def DataReader(
             pause=pause,
             session=session,
         ).read()
-    elif data_source == "robinhood":
-        return RobinhoodHistoricalReader(
-            symbols=name,
-            start=start,
-            end=end,
-            retry_count=retry_count,
-            pause=pause,
-            session=session,
-        ).read()
     elif data_source == "tiingo":
         return TiingoDailyReader(
             symbols=name,
@@ -567,6 +558,18 @@ def DataReader(
             api_key=api_key,
         ).read()
 
+    elif data_source == "av-forex-daily":
+        return AVTimeSeriesReader(
+            symbols=name,
+            function="FX_DAILY",
+            start=start,
+            end=end,
+            retry_count=retry_count,
+            pause=pause,
+            session=session,
+            api_key=api_key,
+        ).read()
+
     elif data_source == "av-daily":
         return AVTimeSeriesReader(
             symbols=name,
@@ -661,6 +664,16 @@ def DataReader(
             session=session,
         ).read()
 
+    elif data_source == "naver":
+        return NaverDailyReader(
+            symbols=name,
+            start=start,
+            end=end,
+            retry_count=retry_count,
+            pause=pause,
+            session=session,
+        ).read()
+
     else:
         msg = "data_source=%r is not implemented" % data_source
         raise NotImplementedError(msg)
